/*
yuvtrack - Tracking a single light object (like ISS)

Copyright (C) 2015 Luca Bonissi <lucabon@scarsita.it>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


char *substr_ret=NULL;

// *** Find a sub-string into a string. The function returns the position of ***
// *** first character, -1 if the sub-string is not found                    ***
int strpos(char s[],char ch[])
{
  int i,length,c,lengthch;

  int start_pos=0;
  length=strlen(s);
  lengthch=strlen(ch);
  if(start_pos>length) return(-1);
  i=start_pos;
  if(lengthch==0) return(start_pos);
  while((i<=length-lengthch))
  {
    while((i<=length-lengthch)&&(s[i]!=ch[0])) i++;
    if(i>length-lengthch) return(-1);
    c=0;
    while((c<lengthch)&&(s[i+c]==ch[c])) c++;
    if(c==lengthch) return(i);
    i++;
  }
  return(-1);
}

// *** Extract a sub-string from input string, startin from start_pos and    ***
// *** fetching nchar characters.                                            ***
char *substr(char s[],int start_pos,int nchar)
{
  int length;
  if(substr_ret!=NULL) free(substr_ret);
  length=strlen(s);
  if(start_pos>=length)
  {
    substr_ret=malloc(1);
    substr_ret[0]=0;
  }
  else
  {
    if((length-start_pos)<nchar) nchar=length-start_pos;
    if(nchar<0) nchar=0;
    substr_ret=malloc(nchar+1);
    memcpy(substr_ret,&(s[start_pos]),nchar);
    substr_ret[nchar]=0;
  }
  return(substr_ret);
}


int main(int argc,char *argv[])
{
// YUV4MPEG2 W768 H576 F25:1 It A1:1
  int width,height,frame_rate,interlaced,int2;
  char inta[255],header[255],frame[255],aspect[255];
  int iW,iH,iF,iD,iI,iA,level=14,c,cd,level2=100,level3=1500;
  FILE *f;
  time_t t;
  char sss[255];
  int outw=640;
  int outh=360;


  level2=1100;
  level3=19300;
  level2=50;
  level3=400;
  int dminlevel=64;
  int margin=24;
  if(argc>1) level2=atoi(argv[1]);
  if(argc>2) level3=atoi(argv[2]);
  unsigned char *buf1,*pbuf1,*pbufu,*pbufv,*bufout,*pbufoutu,*pbufoutv;
  int widthx2,widthd2,row,col,rowc,diff,first=1,x,y,sx,sy;
  int nframe,l;
  long long sumx,sumy,count;

  if(!fgets(header,254,stdin)) {
    perror("ERROR READING HEADER");
    exit(1);
  }
  
  if(strncmp(header,"YUV4MPEG2",9)!=0) {
    perror("INVALID HEADER");
    exit(1);
  }

  iW=strpos(header," W");
  iH=strpos(header," H");
  iF=strpos(header," F");
  iI=strpos(header," I");
  iA=strpos(header," A");

  if(iW<0 || iH<0 || iF<0 || iI<0 || iA<0) {
    perror("INVALID HEADER FORMAT");
    exit(1);
  }

  iW+=2;
  width=atoi(substr(header,iW,iH-iW));
  iH+=2;
  height=atoi(substr(header,iH,iF-iH));
  iF+=2;
  frame_rate=atoi(substr(header,iF,iI-iF-2));
  iI+=2;
  strcpy(inta,substr(header,iI,iA-iI));
  iA+=2;
  strcpy(aspect,substr(header,iA,strlen(header)-1));

  if(inta[0]=='t') interlaced=0;
  else interlaced=1;
  int2=interlaced;

  //printf("YUV4MPEG2 W%d H%d F%d:1 Ip A%s",outw,outh,frame_rate,aspect);
  printf("YUV4MPEG2 W%d H%d %s",outw,outh,substr(header,iF,strlen(header)-iF));
  buf1=malloc(width*height*3/2);
  bufout=malloc(outw*outh*3/2);

  widthx2=width*2;
  widthd2=width/2;
  nframe=0;
  while(fgets(frame,254,stdin)) {
    if(fread(buf1,1,width*height*3/2,stdin)<(width*height)) {
      perror("ERROR READING buffer");
      exit(1);
    }
    pbuf1=buf1;
    sumx=sumy=count=0;
    int minlevel=dminlevel;
    for(y=0;y<height;y++) {
      for(x=0;x<width;x++) {
	if(*pbuf1>minlevel) {
	   l=*pbuf1-minlevel;
	   sumx+=x*l;
	   sumy+=y*l;
	   count+=l;
	}
	pbuf1++;
      }
    }
    if(count>0) {
      sx=(int)((sumx*10/count+5)/10);
      sy=(int)((sumy*10/count+5)/10);
      pbufoutu=bufout+outw*outh;
      pbufoutv=pbufoutu+outw*outh/4;
      pbufu=buf1+width*height;
      pbufv=pbufu+width*height/4;
      memset(bufout,16,outw*outh);
      memset(pbufoutu,128,outw*outh/2);
      int fx,fy,lx,ly,wx;
      fy=sy-outh/2;	if(fy<0) fy=0;	fy=fy-sy+outh/2;
      fx=sx-outw/2;	if(fx<0) fx=0;	fx=fx-sx+outw/2;
      ly=sy+outh/2;	if(ly>=height) ly=height-1;	ly=ly-sy+outh/2;
      lx=sx+outw/2;	if(lx>=width)  lx=width;	lx=lx-sx+outw/2;
      wx=lx-fx;
      if((fx<(outw/2-margin))&&
         (fy<(outh/2-margin))&&
         (lx>(outw/2+margin))&&
         (ly>(outh/2+margin))) {
        fprintf(stderr,"Frame %d, x=%d, y=%d, fx=%d, lx=%d, fy=%d, fx=%d, wx=%d\n",nframe,sx,sy,fx,lx,fy,ly,wx);
        printf(frame);
	for(y=fy;y<ly;y++) {
	  memcpy(bufout+y*outw+fx,buf1+(y+sy-outh/2)*width+(fx+sx-outw/2),wx);
	  if(y%2==0) {
	    // Copy chroma
	    memcpy(pbufoutu+y*outw/4+fx/2,pbufu+(y+sy-outh/2)*width/4+(fx+sx-outw/2)/2,wx/2);
	    memcpy(pbufoutv+y*outw/4+fx/2,pbufv+(y+sy-outh/2)*width/4+(fx+sx-outw/2)/2,wx/2);
	  }
	}
	fwrite(bufout,outw*outh*3/2,1,stdout);
      }
      else {
        fprintf(stderr,"Skip frame %d, x=%d, y=%d, fx=%d, lx=%d, fy=%d, fx=%d, wx=%d\n",nframe,sx,sy,fx,lx,fy,ly,wx);
      }
    }
    else {
      fprintf(stderr,"Skip frame %d (count 0)\n",nframe);
    }
    nframe++;
    // Customization for ISS video
    if(nframe>100) dminlevel=32;
    if(nframe>600) dminlevel=20;
  }
  fprintf(stderr,"END\n");
  free(buf1);
  free(bufout);
}
